@@ -47,9 +47,9 @@ BYT;
 """
         expected = [
             {'number': 1, 'start': 1, 'end': 501, 'size': 500,
-             'filesystem': 'ext4', 'flags': 'boot'},
+             'filesystem': 'ext4', 'partition_name': '', 'flags': 'boot'},
             {'number': 2, 'start': 501, 'end': 476940, 'size': 476439,
-             'filesystem': '', 'flags': ''},
+             'filesystem': '', 'partition_name': '', 'flags': ''},
         ]
         execute_mock.return_value = (output, '')
         result = disk_utils.list_partitions('/dev/fake')
@@ -70,6 +70,46 @@ BYT;
         self.assertEqual(1, log_mock.call_count)
 
 
+@mock.patch.object(utils, 'execute', autospec=True)
+class ListPartitionsGPTTestCase(base.IronicLibTestCase):
+
+    def test_correct(self, execute_mock):
+        output = """
+BYT;
+/dev/vda:40960MiB:virtblk:512:512:gpt:Virtio Block Device:;
+2:1.00MiB:2.00MiB:1.00MiB::Bios partition:bios_grub;
+1:4.00MiB:5407MiB:5403MiB:ext4:Root partition:;
+3:5407MiB:5507MiB:100MiB:fat16:Boot partition:boot, esp;
+"""
+        expected = [
+            {'end': 2, 'number': 2, 'start': 1, 'flags': 'bios_grub',
+             'filesystem': '', 'partition_name': 'Bios partition', 'size': 1},
+            {'end': 5407, 'number': 1, 'start': 4, 'flags': '',
+             'filesystem': 'ext4', 'partition_name': 'Root partition',
+             'size': 5403},
+            {'end': 5507, 'number': 3, 'start': 5407,
+             'flags': 'boot, esp', 'filesystem': 'fat16',
+             'partition_name': 'Boot partition', 'size': 100},
+        ]
+        execute_mock.return_value = (output, '')
+        result = disk_utils.list_partitions('/dev/fake')
+        self.assertEqual(expected, result)
+        execute_mock.assert_called_once_with(
+            'parted', '-s', '-m', '/dev/fake', 'unit', 'MiB', 'print',
+            use_standard_locale=True, run_as_root=True)
+
+    @mock.patch.object(disk_utils.LOG, 'warning', autospec=True)
+    def test_incorrect(self, log_mock, execute_mock):
+        output = """
+BYT;
+/dev/vda:40960MiB:virtblk:512:512:gpt:Virtio Block Device:;
+2:XX1.00MiB:---:1.00MiB::primary:bios_grub;
+"""
+        execute_mock.return_value = (output, '')
+        self.assertEqual([], disk_utils.list_partitions('/dev/fake'))
+        self.assertEqual(1, log_mock.call_count)
+
+
 @mock.patch.object(disk_partitioner.DiskPartitioner, 'commit', lambda _: None)
 class WorkOnDiskTestCase(base.IronicLibTestCase):
 
@@ -388,6 +428,68 @@ class WorkOnDiskTestCase(base.IronicLibTestCase):
                                                     conv_flags='sparse')
 
 
+class GetUEFIDiskIdentifierTestCase(base.IronicLibTestCase):
+
+    def setUp(self):
+        super(GetUEFIDiskIdentifierTestCase, self).setUp()
+        self.dev = '/dev/fake'
+
+    @mock.patch.object(utils, 'execute', autospec=True)
+    def test_get_uefi_disk_identifier_uefi_bootable_image(self, mock_execute):
+        mock_execute.return_value = ('', '')
+        fdisk_output = """
+Disk /dev/sda: 931.5 GiB, 1000171331584 bytes, 1953459632 sectors
+Units: sectors of 1 * 512 = 512 bytes
+Sector size (logical/physical): 512 bytes / 512 bytes
+I/O size (minimum/optimal): 262144 bytes / 262144 bytes
+Disklabel type: gpt
+Disk identifier: 73457A6C-3595-4965-8D83-2EA1BD85F327
+
+Device          Start        End    Sectors   Size Type
+/dev/fake-part1        2048    1050623    1048576   512M EFI System
+/dev/fake-part2     1050624 1920172031 1919121408 915.1G Linux filesystem
+/dev/fake-part3  1920172032 1953458175   33286144  15.9G Linux swap
+"""
+        partition_id = '/dev/fake-part1'
+        lsblk_output = 'UUID="ABCD-B05B"\n'
+        part_result = 'ABCD-B05B'
+        mock_execute.side_effect = [(fdisk_output, ''), (lsblk_output, '')]
+        result = disk_utils.get_uefi_disk_identifier(self.dev)
+        self.assertEqual(part_result, result)
+        execute_calls = [
+            mock.call('fdisk', '-l', self.dev, run_as_root=True),
+            mock.call('lsblk', '-PbioUUID', partition_id,
+                      run_as_root=True)
+        ]
+        mock_execute.assert_has_calls(execute_calls)
+
+    @mock.patch.object(utils, 'execute', autospec=True)
+    def test_get_uefi_disk_identifier_non_uefi_bootable_image(self,
+                                                              mock_execute):
+        mock_execute.return_value = ('', '')
+        fdisk_output = """
+Disk /dev/vda: 50 GiB, 53687091200 bytes, 104857600 sectors
+Units: sectors of 1 * 512 = 512 bytes
+Sector size (logical/physical): 512 bytes / 512 bytes
+I/O size (minimum/optimal): 512 bytes / 512 bytes
+Disklabel type: dos
+Disk identifier: 0xb82b9faf
+
+Device     Boot Start       End   Sectors Size Id Type
+/dev/fake-part1  *     2048 104857566 104855519  50G 83 Linux
+"""
+        partition_id = None
+        mock_execute.side_effect = [(fdisk_output, ''),
+                                    processutils.ProcessExecutionError()]
+        self.assertRaises(exception.InstanceDeployFailure,
+                          disk_utils.get_uefi_disk_identifier, self.dev)
+        execute_calls = [
+            mock.call('fdisk', '-l', self.dev, run_as_root=True),
+            mock.call('lsblk', '-PbioUUID', partition_id, run_as_root=True)
+        ]
+        mock_execute.assert_has_calls(execute_calls)
+
+
 @mock.patch.object(utils, 'execute', autospec=True)
 class MakePartitionsTestCase(base.IronicLibTestCase):
 
@@ -468,7 +570,14 @@ class MakePartitionsTestCase(base.IronicLibTestCase):
         fuser_cmd = ['fuser', 'fake-dev']
         fuser_call = mock.call(*fuser_cmd, run_as_root=True,
                                check_exit_code=[0, 1])
-        mock_exc.assert_has_calls([parted_call, fuser_call])
+
+        sync_calls = [mock.call('sync'),
+                      mock.call('udevadm', 'settle'),
+                      mock.call('partprobe', self.dev, attempts=10,
+                                run_as_root=True),
+                      mock.call('sgdisk', '-v', self.dev, run_as_root=True)]
+
+        mock_exc.assert_has_calls([parted_call, fuser_call] + sync_calls)
 
     def test_make_partitions(self, mock_exc):
         self._test_make_partitions(mock_exc, boot_option="netboot")
@@ -581,7 +690,7 @@ class MakePartitionsTestCase(base.IronicLibTestCase):
         self.assertEqual(expected_result, result)
 
 
-@mock.patch.object(utils, 'execute', autospec=True, return_value=('', ''))
+@mock.patch.object(utils, 'execute', autospec=True)
 class DestroyMetaDataTestCase(base.IronicLibTestCase):
 
     def setUp(self):
@@ -590,9 +699,31 @@ class DestroyMetaDataTestCase(base.IronicLibTestCase):
         self.node_uuid = "12345678-1234-1234-1234-1234567890abcxyz"
 
     def test_destroy_disk_metadata(self, mock_exec):
+        # Note(TheJulia): This list will get-reused, but only the second
+        # execution returning a string is needed for the test as otherwise
+        # command output is not used.
+        mock_exec.side_effect = iter([
+            (None, None),
+            ('1024\n', None),
+            (None, None),
+            (None, None),
+            (None, None),
+            (None, None)])
+
         expected_calls = [mock.call('wipefs', '--force', '--all', 'fake-dev',
                                     run_as_root=True,
                                     use_standard_locale=True),
+                          mock.call('blockdev', '--getsz', 'fake-dev',
+                                    check_exit_code=[0],
+                                    run_as_root=True),
+                          mock.call('dd', 'bs=512', 'if=/dev/zero',
+                                    'of=fake-dev', 'count=33',
+                                    run_as_root=True,
+                                    use_standard_locale=True),
+                          mock.call('dd', 'bs=512', 'if=/dev/zero',
+                                    'of=fake-dev', 'count=33', 'seek=991',
+                                    run_as_root=True,
+                                    use_standard_locale=True),
                           mock.call('sgdisk', '-Z', 'fake-dev',
                                     run_as_root=True,
                                     use_standard_locale=True),
@@ -618,11 +749,26 @@ class DestroyMetaDataTestCase(base.IronicLibTestCase):
         expected_calls = [mock.call('wipefs', '--force', '--all', 'fake-dev',
                                     run_as_root=True,
                                     use_standard_locale=True),
+                          mock.call('blockdev', '--getsz', 'fake-dev',
+                                    check_exit_code=[0],
+                                    run_as_root=True),
+                          mock.call('dd', 'bs=512', 'if=/dev/zero',
+                                    'of=fake-dev', 'count=33',
+                                    run_as_root=True,
+                                    use_standard_locale=True),
+                          mock.call('dd', 'bs=512', 'if=/dev/zero',
+                                    'of=fake-dev', 'count=33', 'seek=991',
+                                    run_as_root=True,
+                                    use_standard_locale=True),
                           mock.call('sgdisk', '-Z', 'fake-dev',
                                     run_as_root=True,
                                     use_standard_locale=True)]
-        mock_exec.side_effect = [(None, None),
-                                 processutils.ProcessExecutionError()]
+        mock_exec.side_effect = iter([
+            (None, None),
+            ('1024\n', None),
+            (None, None),
+            (None, None),
+            processutils.ProcessExecutionError()])
         self.assertRaises(processutils.ProcessExecutionError,
                           disk_utils.destroy_disk_metadata,
                           self.dev,
@@ -630,11 +776,14 @@ class DestroyMetaDataTestCase(base.IronicLibTestCase):
         mock_exec.assert_has_calls(expected_calls)
 
     def test_destroy_disk_metadata_wipefs_not_support_force(self, mock_exec):
-        mock_exec.side_effect = iter(
-            [processutils.ProcessExecutionError(description='--force'),
-             (None, None),
-             (None, None),
-             ('', '')])
+        mock_exec.side_effect = iter([
+            processutils.ProcessExecutionError(description='--force'),
+            (None, None),
+            ('1024\n', None),
+            (None, None),
+            (None, None),
+            (None, None),
+            (None, None)])
 
         expected_call = [mock.call('wipefs', '--force', '--all', 'fake-dev',
                                    run_as_root=True,
@@ -690,6 +839,9 @@ class PopulateImageTestCase(base.IronicLibTestCase):
         self.assertFalse(mock_dd.called)
 
 
+# NOTE(TheJulia): _trigger_device_rescan is systemwide thus pointless
+# to execute in the file test case. Also, CI unit test jobs lack sgdisk.
+@mock.patch.object(disk_utils, '_trigger_device_rescan', lambda *_: None)
 @mock.patch.object(utils, 'wait_for_disk_to_become_available', lambda *_: None)
 @mock.patch.object(disk_utils, 'is_block_device', lambda d: True)
 @mock.patch.object(disk_utils, 'block_uuid', lambda p: 'uuid')
@@ -847,6 +999,25 @@ class OtherFunctionTestCase(base.IronicLibTestCase):
                           disk_utils.is_block_device, device)
         mock_os.assert_has_calls([mock.call(device)] * 3)
 
+    @mock.patch.object(os, 'stat', autospec=True)
+    def test_is_block_device_attempts(self, mock_os):
+        CONF.set_override('partition_detection_attempts', 2,
+                          group='disk_utils')
+        device = '/dev/disk/by-path/ip-1.2.3.4:5678-iscsi-iqn.fake-lun-9'
+        mock_os.side_effect = OSError
+        self.assertRaises(exception.InstanceDeployFailure,
+                          disk_utils.is_block_device, device)
+        mock_os.assert_has_calls([mock.call(device)] * 2)
+
+    @mock.patch.object(os, 'stat', autospec=True)
+    def test_is_block_device_deprecated_attempts(self, mock_os):
+        CONF.set_override('iscsi_verify_attempts', 4, group='disk_utils')
+        device = '/dev/disk/by-path/ip-1.2.3.4:5678-iscsi-iqn.fake-lun-9'
+        mock_os.side_effect = OSError
+        self.assertRaises(exception.InstanceDeployFailure,
+                          disk_utils.is_block_device, device)
+        mock_os.assert_has_calls([mock.call(device)] * 4)
+
     @mock.patch.object(imageutils, 'QemuImgInfo', autospec=True)
     @mock.patch.object(os.path, 'exists', return_value=False, autospec=True)
     def test_qemu_img_info_path_doesnt_exist(self, path_exists_mock,
@@ -1330,7 +1501,7 @@ class WholeDiskConfigDriveTestCases(base.IronicLibTestCase):
                         self.node_uuid)
             expected_part = '%s-part4' % self.dev
         elif is_nvme_device:
-            self.dev = '/dev/nvmefake'
+            self.dev = '/dev/nvmefake0'
             expected_part = '%sp4' % self.dev
         else:
             expected_part = '/dev/fake4'
